package cz.mendelu.dnaAnalyser.batch

import cz.mendelu.dnaAnalyser.sequence.ModelSequenceService
import cz.mendelu.dnaAnalyser.sequence.SequenceRepository
import cz.mendelu.dnaAnalyser.sequence.SequenceService
import cz.mendelu.dnaAnalyser.sequence.inport.ImportSequenceService
import cz.mendelu.dnaAnalyser.user.User
import cz.mendelu.dnaAnalyser.utils.controller.Response
import cz.mendelu.dnaAnalyser.utils.model.Identifiable
import cz.mendelu.dnaAnalyser.utils.model.MockIdentifiable
import cz.mendelu.dnaAnalyser.utils.model.Owned
import spock.lang.Specification
import spock.lang.Subject

import java.util.function.Consumer
import java.util.function.UnaryOperator

class BatchServiceTest extends Specification {

    @Subject BatchService batchService

    BatchRepository batchRepository
    User user
    Identifiable identifiable

    SequenceService sequenceService = new SequenceService()
    ImportSequenceService importSequenceService
    ModelSequenceService modelSequenceService


    def setup() {
        batchService = new BatchService()
        batchService.asynchronous = false
        batchService.batchRepository = batchRepository = Mock(BatchRepository)
        user = Mock(User)
        identifiable = Mock(Identifiable)
        sequenceService.importSequenceService = importSequenceService = Mock(ImportSequenceService)
        sequenceService.modelSequenceService = modelSequenceService = Mock(ModelSequenceService)

    }

    def "test findBatchesFor"() {
        given:
        def identifiable = new MockIdentifiable(id: UUID.randomUUID())
        def batch = new Batch()
        def sequence = new Sequence()

        when:
        def result = batchService.findBatchFor(identifiable)
        sequenceService.findOne(identifiable.id)

        then:
        1 * batchRepository.findBatchByEntityClassAndEntityId(identifiable.class.name, identifiable.id) >> batch
        1 * modelSequenceService.findOne(identifiable.id) >> sequence
        result.isPresent() == false && sequence.isEmpty()

    }

    def "test createBatch"() {
        when:
        def result = batchService.createBatch(user, identifiable)

        then:
        result != null
    }

    def "test batchBuilder.execute().start()"() {
        def uuid = UUID.randomUUID()
        _ * identifiable.getId() >> uuid

        BatchBuilder batchBuilder = batchService.createBatch(user, identifiable)
        def execute = Mock(Consumer)

        when:
        def batch = batchBuilder
                .execute(execute)
                .start()

        then:
        1 * execute.accept(uuid)

        batch.status == Batch.Status.FINISH
    }

    def "test change batch status after Batch failed"() {
        def uuid = UUID.randomUUID()
        _ * identifiable.getId() >> uuid

        BatchBuilder batchBuilder = batchService.createBatch(user, identifiable)
        def execute = Mock(Consumer)

        when:
        def batch = batchBuilder
                .execute(execute)
                .start()

        then:
        1 * execute.accept(uuid) >> {throw new UnsupportedOperationException ("Sth wrong")}

        batch.status == Batch.Status.FAILED
    }
}
