package cz.mendelu.dnaAnalyser.jwt


import cz.mendelu.dnaAnalyser.user.User
import org.springframework.security.core.Authentication
import spock.lang.Specification

class JwtAuthenticationUtilsTest extends Specification {
    def "test toUser"() {
        given:
        def user = new User()
        def authentication = Mock(Authentication)
        authentication.getPrincipal() >> user

        when:
        def result = JwtAuthenticationUtils.toUser(authentication)

        then:
        result == user
    }

    def "test toUser, Authentication.principal not contains"() {
        given:
        def authentication = Mock(Authentication)
        authentication.getPrincipal() >> null

        when:
        JwtAuthenticationUtils.toUser(authentication)

        then:
        thrown(JwtAuthenticationException)
    }

    def "test toUser, Authentication.principal is unsupported type."() {
        given:
        def authentication = Mock(Authentication)
        authentication.getPrincipal() >> 'Not user class'

        when:
        JwtAuthenticationUtils.toUser(authentication)

        then:
        thrown(JwtAuthenticationException)
    }
}
