package cz.mendelu.dnaAnalyser.jwt

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.boot.test.web.client.TestRestTemplate
import org.springframework.http.HttpEntity
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpMethod
import org.springframework.http.HttpStatus
import org.springframework.test.context.ActiveProfiles
import spock.lang.Ignore
import spock.lang.Specification

import static org.springframework.boot.test.context.SpringBootTest.WebEnvironment.RANDOM_PORT

@Ignore
@ActiveProfiles("test")
@SpringBootTest(webEnvironment = RANDOM_PORT)
class JwtControllerTest extends Specification {

    @Autowired
    TestRestTemplate restTemplate

    @Value('${security.jwt.header}')
    private String tokenHeader

    def "PUT:/api/jwt Create JWT AuthenticationToken for existing user"() {
        setup:
        HttpEntity<JwtController.LoginRequest>request = new HttpEntity<>(new JwtController.LoginRequest(login: 'admin@mendelu.cz', password: 'admin'))

        when:
        def response = restTemplate.exchange('/api/jwt', HttpMethod.PUT, request, String.class)

        then:
        response.status == HttpStatus.CREATED.value()
        response.body != null && !response.body.isEmpty()
    }

    def "GET:/api/jwt Refresh JWT AuthenticationToken"() {
        setup:
        def token = restTemplate.exchange('/api/jwt', HttpMethod.POST, null, String.class).body
        def headers = new HttpHeaders()
        headers.add(tokenHeader, token)
        def request = new HttpEntity(null, headers)

        when:
        def response = restTemplate.exchange('/api/jwt', HttpMethod.GET, request, String.class)

        then:
        response.status == HttpStatus.OK.value()
        response.body != null && !response.body.isEmpty()
        response.body != token
    }

    def "POST:/api/jwt Create JWT AuthenticationToken for host"() {
        when:
        def response = restTemplate.exchange('/api/jwt', HttpMethod.POST, null, String.class)

        then:
        response.status == HttpStatus.CREATED.value()
        response.body != null && !response.body.isEmpty()
    }
}
