package cz.mendelu.dnaAnalyser.utils.service

import cz.mendelu.dnaAnalyser.utils.exception.NotFoundException
import cz.mendelu.dnaAnalyser.utils.repository.BaseRepository
import cz.mendelu.dnaAnalyser.email.model.TestModel
import spock.lang.Specification
import spock.lang.Subject

class ModelServiceTest extends Specification {

    @Subject TestModelService service

    def repository = Mock(BaseRepository)

    def setup() {
        service = new TestModelService(repository)
    }

    def "Delegate to repository method findAll()"() {
        when:
            service.findAll(null)

        then:
            1 * repository.findAll(null)
    }

    def "Delegate to repository method findOne()"() {
        setup:
        def id = UUID.randomUUID()

        when:
        service.findOne(id)

        then:
        1 * repository.findById(id) >> Optional.of(new TestModel())
    }

    def "Delegate to repository method findOne() but entity not found"() {
        setup:
        def id = UUID.randomUUID()

        when:
        service.findOne(id)

        then:
        1 * repository.findById(id) >> Optional.empty()
        thrown(NotFoundException)
    }

    def "Delegate to repository method save()"() {
        setup:
        def model = new TestModel()

        when:
        service.save(model)

        then:
        1 * repository.save(model)
    }

    def "Delegate to repository method delete()"() {
        setup:
        def id = UUID.randomUUID()
        def model = new TestModel()

        when:
        service.delete(id)

        then:
        1 * repository.findById(id) >> Optional.of(model)
        1 * repository.delete(model)
    }

    def "Get model generic type class"() {
        when:
        def mClass = service.modelClass()

        then:
        TestModel.class == mClass
    }


    static class TestModelService extends ModelService<TestModel> {

        private final BaseRepository<TestModel, UUID> repository

        TestModelService(BaseRepository<TestModel, UUID> repository) {
            this.repository = repository
        }

        @Override
        protected BaseRepository<TestModel, UUID> getRepository() {
            return repository
        }
    }


}
